/* eslint-disable */

import * as fs from "node:fs";
import path from "node:path";
// @ts-ignore
import read from "read-file";

type Metadata = {
  /**
   * The title of the documentation page.
   */
  title: string;

  /**
   * The description of the documentation page.
   */
  description?: string;
};

/**
 * The directory where the documentation files are stored.
 */
const documentationDirectory = path.join(process.cwd(), "documentation");

/**
 * Gets all the documentation files recursively.
 *
 * @param dirPath the directory path to search for documentation files.
 */
function getDocumentationFiles(dirPath: string): string[] {
  let files: string[] = [];
  const items = fs.readdirSync(dirPath);

  items.forEach(item => {
    const itemPath = path.join(dirPath, item);
    const stat = fs.statSync(itemPath);

    if (stat.isDirectory()) {
      // Recursively traverse directories
      files.push(...getDocumentationFiles(itemPath));
    } else if (stat.isFile() && path.extname(item) === ".md") {
      // Collect markdown files
      files.push(itemPath);
    }
  });

  return files;
}

/**
 * Gets the content of a documentation file.
 *
 * @param file the file to get the content of.
 */
function getDocumentationFileContent(file: string) {
  return read.sync(file, "utf8");
}

/**
 * Gets all the documentation pages.
 */
export function getDocumentation() {
  const files = getDocumentationFiles(documentationDirectory);

  return files.map(file => {
    const { metadata, content } = parseFrontmatter(getDocumentationFileContent(file));
    let slug = path.relative(documentationDirectory, file).replace(/\.(md)$/, "");
    slug = slug.replace(/\\/g, "/"); // Normalize path separators

    return {
      metadata,
      content,
      slug,
    };
  });
}

/**
 * Parses the frontmatter of a file.
 *
 * @param fileContent the content of the file.
 */
function parseFrontmatter(fileContent: string) {
  let frontmatterRegex = /---\s*([\s\S]*?)\s*---/;
  let match = frontmatterRegex.exec(fileContent);
  let frontMatterBlock = match![1];
  let content = fileContent.replace(frontmatterRegex, "").trim();
  let frontMatterLines = frontMatterBlock.trim().split("\n");
  let metadata: Partial<Metadata> = {};

  frontMatterLines.forEach(line => {
    let [key, ...valueArr] = line.split(": ");
    let value = valueArr.join(": ").trim();
    value = value.replace(/^['"](.*)['"]$/, "$1"); // Remove quotes
    metadata[key.trim() as keyof Metadata] = value;
  });

  return { metadata: metadata as Metadata, content };
}
